#include <iostream>
#include "3dengfx/3dengfx.hpp"
#include "common/timer.h"

bool init();
void clean_up();
void update_gfx();
void key_handler(int key);
void bn_handler(int bn, int pressed, int x, int y);
void mouse_handler(int x, int y);

Scene scene;
ntimer timer;
ParticleSystem *p;
TargetCamera *cam;

MotionController cntl[3];

const char *psys_file = "test.psys";
int screenx, screeny;
float emm_x = 0.0, emm_y = 0.0;
bool direct_control = false;

int main(int argc, char **argv) {

	if(argc > 1) {
		psys_file = argv[1];
	}
	
	if(!init()) {
		return -1;
	}

	return fxwt::main_loop();
}

bool init() {
	GraphicsInitParameters *gip = load_graphics_context_config("3dengfx.conf");
	if(!gip) return false;
	
	if(!create_graphics_context(*gip)) {
		return false;
	}

	screenx = gip->x;
	screeny = gip->y;

	fxwt::set_display_handler(update_gfx);
	fxwt::set_idle_handler(update_gfx);
	fxwt::set_keyboard_handler(key_handler);
	fxwt::set_button_handler(bn_handler);
	fxwt::set_motion_handler(mouse_handler);
	atexit(clean_up);

	Light *light = new PointLight(Vector3(50, 100, -100));
	scene.add_light(light);

	cam = new TargetCamera(Vector3(0, 0, -32), Vector3(0, 0, 0));
	scene.add_camera(cam);
	
	/*
	Object *plane = new ObjPlane(Vector3(0, 1, 0), Vector2(500, 500), 10);
	plane->rotate(Vector3(-half_pi, 0, 0));
	plane->get_material_ptr()->diffuse_color = Color(0.5f, 0.5f, 0.5f);

	scene.add_object(plane);
	*/

	p = new ParticleSystem(psys_file);
	//p->set_position(Vector3(0, 10.0, 0));
	
	cntl[0] = MotionController(CTRL_SIN, TIME_WRAP);
	cntl[0].set_sin_func(2.0, 8.0);
	cntl[0].set_control_axis(CTRL_X);
	p->add_controller(cntl[0], CTRL_TRANSLATION);

	cntl[1] = cntl[0];
	cntl[1].set_control_axis(CTRL_Z);
	cntl[1].set_controller_type(CTRL_COS);
	p->add_controller(cntl[1], CTRL_TRANSLATION);

	cntl[2] = cntl[1];
	cntl[2].set_control_axis(CTRL_Y);
	cntl[2].set_sin_func(1.0, 5.0);
	p->add_controller(cntl[2], CTRL_TRANSLATION);

	scene.add_particle_sys(p);

	//scene.render_sequence(0, 5000, 24);
	//return false;

	timer_reset(&timer);
	timer_start(&timer);
	return true;
}

void update_gfx() {
	unsigned long msec = timer_getmsec(&timer);

	if(direct_control) {
		p->set_position(Vector3(emm_x, emm_y, 0.0));
	}
	
	scene.render(msec);
	flip();
}

void clean_up() {
	destroy_graphics_context();
}

void key_handler(int key) {
	static bool stopped = false;
	
	switch(key) {
	case 'a':
		cam->zoom(0.8);
		break;

	case 'z':
		cam->zoom(1.2);
		break;
		
	case ' ':
		if(stopped) {
			timer_start(&timer);
			stopped = false;
		} else {
			timer_stop(&timer);
			stopped = true;
		}
		break;

	case '\n':
	case '\r':
		direct_control = !direct_control;
		if(direct_control) {
			std::vector<MotionController> *vec = p->get_controllers(CTRL_TRANSLATION);
			vec->erase(vec->begin(), vec->end());
		} else {
			p->add_controller(cntl[0], CTRL_TRANSLATION);
			p->add_controller(cntl[1], CTRL_TRANSLATION);
			p->add_controller(cntl[2], CTRL_TRANSLATION);
			p->set_position(Vector3(0, 0, 0));
		}
		break;

	case '`':
		screen_capture();
		break;
	
	case 27:
		exit(0);

	default:
		break;
	}
}

static int left_pressed = 0;

void bn_handler(int bn, int pressed, int x, int y) {
	if(bn == fxwt::BN_LEFT) {
		left_pressed = pressed;
		//SDL_ShowCursor(!pressed);
	}
}

void mouse_handler(int x, int y) {
	if(left_pressed) {
		emm_x = (((float)x / (float)screenx) * 2.0 - 1.0) * 20.0;
		emm_y = -(((float)y / (float)screeny) * 2.0 - 1.0) * 20.0;
	}
}
